from ._version import VERSION
from .challenges.dragonfind import dragonfind_10_to_500  # noqa
from .challenges.egg import eggdrop_100floor_2egg  # noqa
from .challenges.minors import (  # noqa
    largest_ascending_num,
    largest_even_divided,
    largest_multiple,
    last_with_three_multiple_difference,
    letter_occurrences,
    longest_with_three_same_letters,
    smallest_where_double_also,
    string_with_most_a_letters,
    sum_odd_positives,
    word_with_most_of_letters,
)

__version__ = VERSION
